/* (C) 2022 YiRing, Inc. */
package com.yiring.auth.service.zy.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.http.Header;
import cn.hutool.http.HttpRequest;
import cn.hutool.json.JSONUtil;
import com.yiring.auth.service.zy.ZyHttpService;
import com.yiring.auth.util.ZyUtil;
import com.yiring.common.core.Result;
import java.io.Serializable;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * ZyHttp请求
 * @author LJ-2204
 * @date 2022/4/18
 */

@Transactional(rollbackFor = RuntimeException.class)
@Service
@Slf4j
public class ZyHttpServiceImpl implements ZyHttpService {

    public final String TOKEN_TYPE = "bearer ";

    public final String URL_PREFIX = "http://project.yz-online.com:789";

    @Override
    @Async("httpThreadPool")
    public CompletableFuture<Result<Serializable>> get(String urlString, Object obj, int timeout) {
        String json = JSONUtil.toJsonStr(obj);
        String token = ZyUtil.login();
        String body = HttpRequest
            .get(URL_PREFIX + urlString)
            .header(Header.AUTHORIZATION, TOKEN_TYPE + token)
            .header(Header.CONTENT_TYPE, "application/json")
            .body(json)
            .timeout(timeout)
            .execute()
            .body();

        return CompletableFuture.completedFuture(getResult(body));
    }

    @Override
    @Async("httpThreadPool")
    public CompletableFuture<Result<Serializable>> post(String urlString, Object obj, int timeout) {
        String json = JSONUtil.toJsonStr(obj);
        String token = ZyUtil.login();
        String body = HttpRequest
            .post(URL_PREFIX + urlString)
            .header(Header.AUTHORIZATION, TOKEN_TYPE + token)
            .header(Header.CONTENT_TYPE, "application/json")
            .body(json)
            .timeout(timeout)
            .execute()
            .body();

        return CompletableFuture.completedFuture(getResult(body));
    }

    @Override
    @Async("httpThreadPool")
    public CompletableFuture<Result<Serializable>> put(String urlString, Object obj, int timeout) {
        String json = JSONUtil.toJsonStr(obj);
        String token = ZyUtil.login();
        String body = HttpRequest
            .put(URL_PREFIX + urlString)
            .header(Header.AUTHORIZATION, TOKEN_TYPE + token)
            .header(Header.CONTENT_TYPE, "application/json")
            .body(json)
            .timeout(timeout)
            .execute()
            .body();

        return CompletableFuture.completedFuture(getResult(body));
    }

    @Override
    @Async("httpThreadPool")
    public CompletableFuture<Result<Serializable>> delete(String urlString, Object obj, int timeout) {
        String json = JSONUtil.toJsonStr(obj);
        String token = ZyUtil.login();
        String body = HttpRequest
            .delete(URL_PREFIX + urlString)
            .header(Header.AUTHORIZATION, TOKEN_TYPE + token)
            .header(Header.CONTENT_TYPE, "application/json")
            .body(json)
            .timeout(timeout)
            .execute()
            .body();

        return CompletableFuture.completedFuture(getResult(body));
    }

    /**
     * 返回类型转换
     * @param body json
     * @return Result
     */
    private static Result<Serializable> getResult(String body) {
        String data = null;
        Map map = JSONUtil.toBean(body, Map.class);
        Integer code = Convert.toInt(map.get("code"));
        String msg = Convert.toStr(map.get("msg"));
        if (ObjectUtil.isNotEmpty(map.get("data"))) {
            data = Convert.toStr(map.get("data"));
        }
        return Result.builder().status(code).message(msg).body(data).build();
    }
}
